'''
mainwindow.setWindowFlags(QtCore.Qt.FramelessWindowHint)
mainwindow.showFullScreen()
'''
import sys,os
# Import PySide classes
from PySide.QtCore import *
from PySide.QtGui import *
from mainwindow import Ui_MainWindow
from error_dialog import Ui_Error_Dialog
from reboot_dialog import Ui_Reboot_dialog

ADMINISTRACE_BASE='http://centrum-develop-dboff.cloudtoprivate.cz'

task_val = 0
overall_val = 0


class MainWindow(QMainWindow, Ui_MainWindow):
	sig_status = Signal(dict)
	sig_void = Signal()
	sig_str = Signal(str)
	
	sig_error = Signal(dict)
	sig_reboot = Signal()
	
	def __init__(self, parent=None):
		super(MainWindow, self).__init__(parent)
		self.setupUi(self)
		self.hide_all()
		self.progress_current_action = None
		self.w_caption.show()
		
		self.error_dialog = Error_Dialog(self)
		self.reboot_dialog = Reboot_dialog(self)
		self.sig_error.connect(self.show_error_dialog)
		self.sig_reboot.connect(self.show_reboot_dialog)

	def hide_all(self):	
		self.l_debug.hide()
		self.w_caption.hide()
		self.w_progress.hide()
		self.w_before_install.hide()
		
	@Slot(dict)
	def update_progress(self,installation_status):
		self.l_progress_task.setText('Current task: {}'.format(installation_status['action']))
		if installation_status['action'] != self.progress_current_action:
			self.p_progress_overall.setValue(self.p_progress_overall.value()+1)
			self.progress_current_action = installation_status['action']
		if not installation_status.has_key('progress'):
			pass
		elif installation_status['progress'] == None:
			self.p_progress_task.setValue(0)
			self.p_progress_task.setMaximum(0)
		else:
			self.p_progress_task.setMaximum(installation_status['progress']['total'])
			self.p_progress_task.setValue(installation_status['progress']['current'])
		
		self.l_debug.setText(str(installation_status))
		
	@Slot()
	def close_window():
		self.close()
		
	@Slot(str)
	def show_before_install(self,server_ip):
		self.hide_all()
		self.l_ip.setText(server_ip)
		self.w_before_install.show()
	
	@Slot()
	def show_progress(self):
		self.hide_all()
		self.l_progress_task.setText('')
		self.p_progress_task.setValue(0)
		self.l_progress_overall.setText('Installation progress:')
		self.p_progress_overall.setMaximum(11)
		self.p_progress_overall.setValue(0)
		self.w_progress.show()
	
	@Slot(dict)	
	def show_error_dialog(self, data):
		self.error_dialog.settings(data)
		self.error_dialog.exec_()
	
	@Slot()	
	def show_reboot_dialog(self):
		self.reboot_dialog.exec_()

class Error_Dialog(QDialog, Ui_Error_Dialog):
	close_sig = Signal()
	def __init__(self, parent = None):
		super(Error_Dialog, self).__init__(parent)
		self.setupUi(self)
		self._buttons = []
		self._data = {}

	def settings(self, data):
		self._data = data
		if('title' in data):
			self.error_title.setText(data['title'])
		
		if('simple' in data and data['simple']):
			self.error_desc.hide()
			self.error_desc_label.show()
			self.resize(399, 162)
			self.button_container.setGeometry(QRect(10, 120, 381, 31))
			if('desc' in data):
				self.error_desc_label.setText(data['desc'])
				
		else:
			self.error_desc_label.hide()
			self.error_desc.show()
			self.resize(399, 292)
			self.button_container.setGeometry(QRect(10, 250, 381, 31))
			if('desc' in data):
				self.error_desc.setPlainText(data['desc'])
			
		
		while len(self._buttons):
			button = self._buttons.pop()
			del button
			
		if('type' not in data):
			self.add_button('OK', True).clicked.connect(self.accept)
			
		else:
			if(data['type'] == 'exception'):
				#from left
				self.add_button('Reboot').clicked.connect(self.reboot)
				self.add_button('Send report').clicked.connect(self.send_error_report)
				
			elif(data['type'] == 'system_error'):
				#from left
				self.add_button('Reboot').clicked.connect(self.reboot)
				
			else:
				self.add_button('OK', True).clicked.connect(self.accept)
			
	def add_button(self, text, default = False):
		button = QPushButton(self.button_container)
		button.setText(text)
		self._buttons.append(button)
		position = int(self.button_container.geometry().width()) - reduce(lambda x, y: x + y + 5, map(lambda x: int(x.geometry().width()), self._buttons), -5)
		geometry = button.geometry()
		geometry.moveLeft(position)
		geometry.moveTop(5)
		geometry.setHeight(21)
		button.setGeometry(geometry)
		button.setDefault(default)
		return button
		
	@Slot()
	def send_error_report(self):
		from c2pinstaller import remote_api
		api = remote_api.RemoteApi(ADMINISTRACE_BASE+'/cloud_api/_call')
		if('simple' in data):
			del data['simple']
		print api.send_error_report(self._data)
	
	@Slot()
	def reboot(self):
		os.system('reboot')
		
		
class Reboot_dialog(QDialog, Ui_Reboot_dialog):
	def __init__(self, parent = None):
		super(Reboot_dialog, self).__init__(parent)
		self.setupUi(self)
		self.pushButton.clicked.connect(self.set_reboot)
	
	@Slot()
	def set_reboot(self):
		from c2pinstaller import config
		config.reboot_button_pressed = True
		
class Gui(object):
	"""docstring for Gui"""
	def __init__(self,argv):
		self.app = QApplication(argv)
		self.app.setStyle("Plastique")
		self.frame = MainWindow()
		if '--window' in argv:
			self.frame.show()
		else:
			self.frame.setWindowFlags(Qt.FramelessWindowHint)
			self.frame.showFullScreen()
		
	def exec_(self):
		self.app.exec_()
		return
		
	def show_error_dialog(self, data):
		self.frame.sig_error.emit(data)
		
	def show_reboot_dialog(self):
		self.frame.sig_reboot.emit()
		
	def update_progress(self,installation_status):
		"""docstring for update_progress"""
		self.frame.sig_status.connect(self.frame.update_progress)
		self.frame.sig_status.emit(installation_status)
		self.frame.sig_status.disconnect(self.frame.update_progress)
	
	
	def show_installation_start(self,server_ip):
		"""docstring for show_installation_start"""
		self.frame.sig_str.connect(self.frame.show_before_install)
		self.frame.sig_str.emit(server_ip)
		self.frame.sig_str.disconnect(self.frame.show_before_install)
		pass
	
	def installation_started(self):
		"""docstring for installation_started"""
		self.frame.sig_void.connect(self.frame.show_progress)
		self.frame.sig_void.emit()
		self.frame.sig_void.disconnect(self.frame.show_progress)
		pass
	
	def close(self):	
		self.frame.sig_void.connect(self.frame.close_window)
		self.frame.sig_void.emit()
		self.frame.sig_void.disconnect(self.frame.close_window)
		pass