#!/usr/bin/python

import sys, os, getopt,re,shlex
r_line = re.compile('^(.*?)(#|$)')

def time_mirrorlist(url):
	import pycurl,time
	from StringIO import StringIO
	import xml.etree.ElementTree as ET
	
	repourl = ''
	
	buf = StringIO()
	c = pycurl.Curl()
	c.setopt(c.URL, url)
	c.setopt(c.WRITEFUNCTION, buf.write)
	c.perform()
	c.close()
	
	val = buf.getvalue()
	if val[:5] == '<?xml':
		ns = {'metalink':'http://www.metalinker.org/'}
		xml = ET.fromstring(val)
		repolist = []
		for repo in xml.findall('./metalink:files/metalink:file[0]/metalink:resources/metalink:url[@protocol="http"][@location="CZ"]', ns):
			repolist.append(repo.text[:-len('repodata/repomd.xml')])
	else:
		repolist = val.strip().split('\n')
	
	repotime = None
	
	for repo in repolist:
		try:
			buf = StringIO()
			t = time.time()
			c = pycurl.Curl()
			c.setopt(c.URL, repo+'repodata/repomd.xml')
			c.setopt(c.WRITEFUNCTION, buf.write)
			c.perform()
			c.close()
			buf.close()
			tt = time.time() - t
			if repotime == None or repotime > tt:
				repourl = repo
				repotime = tt
		except:
			pass
	return repourl

def parseFile(path):
	if path.startswith(('https://','http://','ftp://')):
		import pycurl,time
		from StringIO import StringIO
		buf = StringIO()
		c = pycurl.Curl()
		c.setopt(c.URL, path)
		c.setopt(c.WRITEFUNCTION, buf.write)
		c.setopt(c.FOLLOWLOCATION, True)
		c.perform()
		if c.getinfo(c.RESPONSE_CODE) != 200:
			raise Exception('unable to get remote KS. Return code: %d' % (c.getinfo(c.RESPONSE_CODE),))
		c.close()
		buf.seek(0)
		fd = buf
	else:
		fd = open(path)
	ret = parse(fd)
	fd.close()
	return ret

def parseString(my):
	from StringIO import StringIO
	fd = StringIO(my)
	ret = parse(fd)
	fd.close()
	return ret
	
def parse(fd):
	open('/root/kickstart.cfg','w').write(fd.read())
	fd.seek(0)
	config = {'reboot':False, 'packages':set(), 'pre':[], 'post':[], 'fstab':''}
	state = None
	state_opts = {}
	state_buffer = ''
	for line in fd:
		if state != None:
			# we are parsing some special section
			if line.strip() == '%end':
				if state == 'packages':
					for p in state_buffer.strip().split('\n'):
						match_line = r_line.match(p.strip())
						pkg = match_line.group(1)
						if pkg.strip() != '':
							config['packages'].add(pkg.strip())
				elif state in ['pre','post']:
					config[state].append(
						{'opts':state_opts,'buffer':state_buffer}
					)
				elif state in ['fstab']:
					config[state] = state_buffer
				else:
					raise Exception('Unknown special section: '+state)
				state = None
			else:
				state_buffer += line
		else:
			match_line = r_line.match(line.strip())
			if match_line == None:
				raise Exception('invalid line: '.line)
			else:
				args = shlex.split(match_line.group(1))
				if args == []:
					continue
				if args[0][0] == '%':
					state = args[0][1:]
					state_opts = {}
					copt,carg = getopt.getopt(args[1:], '', ['log=','nochroot'])
					for opt in copt:
						state_opts[opt[0][2:]] = opt[1]
					state_buffer = ''
					continue
				else:
					if args[0] in ['url','repo']:
						if 'repo' not in config:
							config['repo'] = []
						copt,carg = getopt.getopt(args[1:], '', ['mirrorlist=','name=','baseurl=','cost='])
						repo = {}
						if args[0] == 'url':
							repo['name'] = 'Base'
						for opt in copt:
							repo[opt[0][2:]] = opt[1].strip('"')
						if 'name' not in repo or ('mirrorlist' not in repo and 'baseurl' not in repo):
							raise Exception('missing repo information')
						if 'mirrorlist' in repo:
							# repo['baseurl'] = time_mirrorlist(repo['mirrorlist'])
							pass
						config['repo'].append(repo)
					elif args[0] == 'reboot':
						config['reboot'] = True
					elif args[0] in ['clearpart','part','ignoredisk','raid','volgroup','logvol']:
						# ignore disk partitioning for now
						continue
					elif args[0] in ['firstboot']:
						# ignore firstboot
						continue
					elif args[0] == 'keyboard':
						copt,carg = getopt.getopt(args[1:], '', ['vckeymap=','xlayouts='])
						config['keyboard'] = {}
						for opt in copt:
							config['keyboard'] = opt[1]
					elif args[0] == 'auth':
						copt,carg = getopt.getopt(args[1:], '', ['enableshadow','passalgo='])
						config['auth'] = {}
						for opt in copt:
							config['auth'] = opt[1]
					elif args[0] == 'lang':
						config['lang'] = args[1]
					elif args[0] == 'timezone':
						config['timezone'] = {
							'name':args[1]
						}
						if '--isUtc' in args:
							config['timezone']['utc'] = True
						else:
							config['timezone']['utc'] = False
					elif args[0] == 'rootpw':
						copt,carg = getopt.getopt(args[1:], '', ['iscrypted'])
						rootpw = {}
						for opt in copt:
							rootpw[opt[0][2:]] = opt[1]
						if 'iscrypted' in rootpw:
							rootpw['iscrypted'] = True
						else:
							rootpw['iscrypted'] = False
						rootpw['value'] = carg[0]
						config['rootpw'] = rootpw
					elif args[0] == 'bootloader':
						copt,carg = getopt.getopt(args[1:], '', ['location=','boot-drive=','append='])
						bootloader = {}
						for opt in copt:
							bootloader[opt[0][2:]] = opt[1]
						config['bootloader'] = bootloader
					# simple command
					else:
						print args
	return config

if __name__ == '__main__':
	import pprint
	pp = pprint.PrettyPrinter(indent=2)
	fd = open(sys.argv[1])
	pp.pprint(parse(fd))
	fd.close()
