import os, subprocess, re, socket, select, struct, time, uuid

def get_server_config():
	server_data = {'public':False}
	
	server_data['disk'] = get_drive_config()
	server_data['meminfo'] = get_meminfo()
	server_data['interfaces'] = get_network_config()
	server_data['interface_mapping'] = get_network_mapping()
	server_data['cpu'] = get_cpuinfo()
	server_data['visible_localities'] = get_visible_localities()
	
	return server_data

def get_visible_localities():
	sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
	sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
	sock.bind(('239.74.234.5', 22769))
	mreq = struct.pack("4sl", socket.inet_aton('239.74.234.5'), socket.INADDR_ANY)
	sock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, mreq)
	now = time.time()

	localities = []
	while (now + 3) > time.time():
		rs, dummy, dummy = select.select([sock],[],[],0.1)
		if(len(rs) == 0):
			continue
		
		data = sock.recv(4096)
		data = struct.unpack('<16s4s20sQ',data)
		locality = uuid.UUID(bytes=data[0]).fields[5]>>24
		locality = hex(int(locality))[2:].rjust(6, '0')
		if(locality not in localities):
			localities.append(locality)
		
	sock.close()
	
	return localities

def get_drive_config():
	"""docstring for get_drive_config"""
	partitions = map(lambda x: x.split(),open('/proc/partitions').read().strip().split('\n')[2:])
	partitions = map(lambda x: x + [get_drive_alignment(x[3])], partitions)

	partitions_out = []
	for part in partitions:
		blkinfo = []
		blkid = []
		if part[4] == False:
			continue
		blkinfo = subprocess.check_output(['lsblk','-n','/dev/%s' % (part[3],)]).split()
		if blkinfo[5] != 'disk':
			continue
		try:
			blkid = subprocess.check_output(['blkid','/dev/%s' % (part[3],)]).split()
			if 'TYPE="iso9660"' in blkid:
				continue
		except:
			pass
		part.append(blkinfo)
		part.append(blkid)
		partitions_out.append(part)
	return partitions_out

def get_meminfo():
	meminfo = open('/proc/meminfo').read().strip().split('\n')
	mem = {}
	for line in meminfo:
		line = line.split(':')
		mem[line[0]] = int(line[1].strip('kB').strip())
	return mem
	
def get_cpuinfo():
	cpuinfo = open('/proc/cpuinfo').read().strip().split('\n\n')
	return map(lambda x: dict(map(lambda y: (y.split(':')[0].strip().replace(' ','_'),y.split(':')[1].strip()),x.split('\n'))),cpuinfo)

def get_network_mapping():
	mappings = {}
	for ifname in os.listdir('/sys/class/net'):
		try:
			ret = subprocess.check_output('udevadm test --action=add `udevadm info -q path /sys/class/net/{}` 2>/dev/null | grep ID_NET_NAME_ONBOARD'.format(ifname),shell=True).strip().split('=')
			mappings[ifname] = ret[1]
		except:
			try:
				ret = subprocess.check_output('udevadm test --action=add `udevadm info -q path /sys/class/net/{}` 2>/dev/null | grep ID_NET_NAME_PATH'.format(ifname),shell=True).strip().split('=')
				mappings[ifname] = ret[1]
			except:
				pass
	return mappings
		
def get_network_config():
	ret = subprocess.check_output(['ip','ad','sh'])
	iface = None
	ifaces = []
	for line in ret.strip().split('\n'):
		mymatch = re.match('^[0-9]+:\s([a-z0-9]+):',line)
		if mymatch:
			# next interface
			if iface != None:
				if addr != None:
					iface['addr'].append(addr)
				ifaces.append(iface)
			iface = {'name':mymatch.group(1)}
			sp = line.split()
			iface['flags'] = sp[2].strip('<>')
			for i in range(3,len(sp),2):
				iface[sp[i]] = sp[i+1]
			addr = None
			iface['addr'] = []

		else:
			# parsujeme adresy
			line = line[4:]
			if line[0] != ' ':
				if addr != None:
					iface['addr'].append(addr)
				sp = line.split()
				addr = {'type':sp[0],'addr':sp[1],'flags':[]}
				i = 2
				while i < len(sp):
					if sp[i] in ['scope','brd']:
						addr[sp[i]] = sp[i+1]
						i+=2
					else:
						addr['flags'].append(sp[i])
						i+=1
			else:
				sp = line.split()
				for i in range(0,len(sp),2):
					addr[sp[i]] = sp[i+1]


	if iface != None:
		if addr != None:
			iface['addr'].append(addr)
		ifaces.append(iface)
	return ifaces
	

def get_drive_alignment(device):
	"""docstring for get_drive_alignment"""
	try:
		alignment = {}
		alignment['block_size'] = int(open('/sys/block/%s/queue/physical_block_size' % (device,)).read().strip())
		alignment['min_io_size'] = int(open('/sys/block/%s/queue/minimum_io_size' % (device,)).read().strip())
		alignment['opt_io_size'] = int(open('/sys/block/%s/queue/optimal_io_size' % (device,)).read().strip())
		alignment['offset'] = int(open('/sys/block/%s/alignment_offset' % (device,)).read().strip())
		alignment['rotational'] = int(open('/sys/block/%s/queue/rotational' % (device,)).read().strip())
		return alignment
	except:
		return False