#!/usr/bin/python

import sys, subprocess, os, tempfile, shutil, time,re,sys,threading,signal
import yum,rpm, traceback, tty, termios
from c2pinstaller import remote_api,server_config,cli,config,gui

ADMINISTRACE_BASE='https://admin.cloudtoprivate.com'

def install(_gui=None):
	try:
		if config.cpu_check:
			tmp = subprocess.check_output('cat /proc/cpuinfo |grep -e flags -e vendor_id', shell = True).strip().split('\n')
			cpu_info = {}
			for line in tmp:
				line = map(lambda x: x.strip(), line.split(':', 1))
				if(line[0] == 'flags'):
					line[1] = line[1].split()
				
				cpu_info[line[0]] = line[1]
			
			if('vendor_id' in cpu_info and cpu_info['vendor_id'] == 'GenuineIntel'):
				if('flags' not in cpu_info or 'vmx' not in cpu_info['flags']):
					raise SystemError('unsupported_cpu')
				
			elif('vendor_id' in cpu_info and cpu_info['vendor_id'] == 'AuthenticAMD'):
				if('flags' not in cpu_info or 'svm' not in cpu_info['flags']):
					raise SystemError('unsupported_cpu')
				
			else:
				raise SystemError('unknown_cpu')
		
		api = None

		if config.ks_path != None:
			config.parse_kickstart()
		else:
			# pokud nemame kickstart na commandline, jedeme standard
			import uuid
			from base64 import b64encode

			try:
				if os.path.exists('.uuid'):
					u = uuid.UUID(bytes=open('.uuid').read())
				else:
					raise Exception()
			except:
				u = uuid.uuid4()
				f = open('.uuid','w')
				f.write(u.bytes)
				f.close()

			# stop vgs and raids
			os.system('umount /mnt/target/* /mnt/target 2>/dev/null')
			os.system('vgchange -an; mdadm --stop --scan')
	
			local_ip = subprocess.check_output("ip ad sh | grep 'inet ' | grep 'scope global'",shell=True).split()[1].split('/')[0]
	
	
			server_data = server_config.get_server_config()
			if config.public_installation != False:
				server_data['public'] = True
				server_data['provider_c2p_transaction_id'] = config.public_installation
	
			api = remote_api.RemoteApi(ADMINISTRACE_BASE+'/cloud_api/_call')
			ret = api.installation_start(b64encode(u.bytes),server_data)
			api.use_sid(ret['sid'])
	
			# write redirect file
			fd = open('/var/www/html/index.html','w')	
			fd.write("""
				<html>
				<head>
				<meta http-equiv="refresh" content="0; url=%s/cloud_install/start?uuid=%s" />
				</head>
				</html>
			""" % (ADMINISTRACE_BASE,str(u)))
			fd.close()
	
			if _gui:
				_gui.show_installation_start(local_ip)
	
			print "server ip: %s" % (local_ip,)
			print "configuration sent - waiting for installation instructions..."
			
			while True:
				ret = api.installation_get_params()
				if type(ret) == type({}) and ret['status'] == 'ready_to_install':
					ks = ret['ks']
					break
				time.sleep(1)
			if _gui:
				_gui.installation_started()
			config.parse_kickstart(ks)
		
		installation_status = {'action':'applying_environment','progress':None}
		if api:
			from c2pinstaller.status_updater import StatusUpdater
			status_updater = StatusUpdater(api,gui,installation_status)
			status_updater.start()
		else:
			status_updater = None
		if _gui:
			_gui.update_progress(installation_status)

		if 'lang' in config.ks:
			os.environ['LANG'] = config.ks['lang']
			os.system('localectl set-locale %s' % (config.ks['lang'],))

		if 'keyboard' in config.ks:
			if 'vckeymap' in config.ks['keyboard']:
				os.system('localectl set-keymap --no-convert %s' % (config.ks['keyboard']['vckeymap'],))
			if 'xlayouts' in config.ks['keyboard']:
				os.system('localectl set-x11-keymap --no-convert %s' % (config.ks['keyboard']['xlayouts'],))

		os.system('umount /mnt/target/* /mnt/target 2>/dev/null')
		if not os.path.exists('/mnt/target'):
			os.mkdir('/mnt/target')

		assert len(config.ks['pre']) > 0, "pre script not defined, how should I know where to install?"


		installation_status['action'] = 'running_pre'
		installation_status['progress'] = {'current':0,'total':len(config.ks['pre'])}
		if status_updater:
			status_updater.update_status(installation_status)
		if _gui:
			_gui.update_progress(installation_status)

		for pre in config.ks['pre']:
			prescript = tempfile.NamedTemporaryFile(delete=False)
			prescript.write(pre['buffer'])
			prescript.close()
			ret = os.system('sh {0} >{0}.log 2>&1'.format(prescript.name))
			if ret != 0:
				raise Exception('Pre-script %s returned errorr: %d' % (prescript.name,ret))
			os.unlink(prescript.name)
			
			installation_status['progress']['current'] += 1
			if status_updater:
				status_updater.update_status(installation_status)
			if _gui:
				_gui.update_progress(installation_status)
		

		# repos
		installation_status['action'] = 'configuring_repos'
		installation_status['progress'] = None
		if status_updater:
			status_updater.update_status(installation_status)
		if _gui:
			_gui.update_progress(installation_status)
	
		for repo in config.ks['repo']:
			repofile = open(os.path.join('/etc/yum/repos.d',repo['name']+'.repo'),'w')
			repofile.write('[%s]\n' % (repo['name'].lower().replace(' ','-'),))
			repofile.write('name=%s\ngpgcheck=0\n' % (repo['name'],))
			if 'mirrorlist' in repo:
				repofile.write('mirrorlist=%s\n' % (repo['mirrorlist'],))
			if 'baseurl' in repo:
				repofile.write('baseurl=%s\n' % (repo['baseurl'],))
			repofile.close()

		# force select base packages
		config.ks['packages'].add('@base')
		config.ks['packages'].add('rootfiles')

		# select and install packages
		rpm.addMacro("_dbpath", "/var/lib/rpm")
		yb = yum.YumBase()
		yb.conf.cachedir = '/mnt/target/var/cache/yum'
		# yb.conf.cache = 0 # disable cache
		yb.conf.installroot = '/mnt/target'
		yb.conf.assumeyes = True
		yb.conf.yumvar['releasever'] = '7'
		print "selecting packages..."

		installation_status['action'] = 'selecting_packages'
		installation_status['progress'] = None
		if status_updater:
			status_updater.update_status(installation_status)
		if _gui:
			_gui.update_progress(installation_status)

		installed = yb.rpmdb.returnPackages()
		packages = list(config.ks['packages'])
		packages += ['kernel','kernel-headers','grub2','grub2-tools']
		for package in packages:
			groups = []
			if package.startswith('@^'):
				groups = yb.comps.return_environments(package[2:])
				for group in groups:
					print('selecting {0}'.format(package))
					yb.selectEnvironment(group.groupid)
				continue
			elif package.startswith('@'):
				groups = yb.comps.return_groups(package[1:])
				for group in groups:
					print('selecting {0}'.format(package))
					yb.selectGroup(group.groupid)
				continue
			pkg = yb.pkgSack.returnNewestByNameArch(patterns=[package])[0]
			if pkg in installed:
				print('{0} is already installed, skipping...'.format(package))
			else:
				print('selecting {0}'.format(package))
				yb.install(pkg)

		installation_status['action'] = 'resolving_dependencies'
		installation_status['progress'] = None
		if status_updater:
			status_updater.update_status(installation_status)
		if _gui:
			_gui.update_progress(installation_status)
		print "resolving deps..."
		yb.resolveDeps()

		installation_status['action'] = 'building_transaction'
		installation_status['progress'] = None
		if status_updater:
			status_updater.update_status(installation_status)
		if _gui:
			_gui.update_progress(installation_status)
		print "building transaction..."
		yb.buildTransaction()

		# TODO - osamostatnit download

		print "processing transaction (dowloading packages)..."

		dlpkgs = map(lambda x: x.po, filter(lambda txmbr:
						txmbr.ts_state in ("i", "u"),
						yb.tsInfo.getMembers()))
		
		installation_status['action'] = 'downloading_packages'
		installation_status['progress'] = {'current':0,'total':len(dlpkgs)}
		if status_updater:
			status_updater.update_status(installation_status)
		if _gui:
			_gui.update_progress(installation_status)
		status_count = 1
		for pkg in dlpkgs:
			print "downloading package %s (%d/%d)" % (pkg,status_count,len(dlpkgs))
			
			installation_status['detail'] = {'package':str(pkg),'action':'download'}
			installation_status['progress']['current'] = status_count
			if status_updater:
				status_updater.update_status(installation_status)
			if _gui:
				_gui.update_progress(installation_status)
			yb.downloadPkgs(pkglist=[pkg])
			status_count += 1

		installation_status['action'] = 'installing_packages'
		installation_status['progress'] = None
		if status_updater:
			status_updater.update_status(installation_status)
		if _gui:
			_gui.update_progress(installation_status)
		yb.processTransaction(callback=cli.TransactionCallBack(),rpmTestDisplay=cli.RPMCallBack(), rpmDisplay=cli.RPMCallBack(status_updater=status_updater, _gui=_gui))

		yb.closeRpmDB()
		yb.close()


		installation_status = {'action':'postinstall','progress':None}
		if status_updater:
			status_updater.update_status(installation_status)
		if _gui:
			_gui.update_progress(installation_status)

	
		# write resolv.conf
		resolv = open('/mnt/target/etc/resolv.conf','w')
		if os.path.isfile('/etc/resolv.conf'):
			resolv.write(open('/etc/resolv.conf').read())
		else:
			resolv.write('''
			nameserver 8.8.8.8
			naemserver 8.8.4.4
			''')
		resolv.close()

		# set root password

		if 'rootpw' in config.ks:
			if not config.ks['rootpw']['iscrypted']:
				import crypt
				cryptpw = crypt.crypt(config.ks['rootpw']['value'])
			else:
				cryptpw = config.ks['rootpw']['value']
			os.system('chroot /mnt/target usermod -p \'%s\' root' % (cryptpw,))

		# install kernel and grub

		try:
			boot_uuid=subprocess.check_output("tune2fs -l `cat /proc/mounts  | grep /mnt/target/boot | awk '{print $1}'` | grep UUID",shell=True).split()[-1].strip()
		except:
			boot_uuid=None

		fstab = open('/mnt/target/etc/fstab','w')
		if boot_uuid == None:
			fstab.write(config.ks['fstab'])
		else:
			fstab.write(config.ks['fstab'].replace('%%boot_uuid%%',boot_uuid))
		fstab.close()


		installation_status = {'action':'bootloader','progress':None}
		if status_updater:
			status_updater.update_status(installation_status)
		if _gui:
			_gui.update_progress(installation_status)
		print 'installing bootloader...'
		os.system("chroot /mnt/target sh -c 'mount -t devtmpfs devtmpfs /dev; mount -t proc proc /proc; mount -t sysfs sysfs /sys; udevadm trigger; grub2-mkconfig -o /boot/grub2/grub.cfg; grub2-install /dev/sda'")

		print 'configuring bootloader...'
		# zajistit vsechny device potrebne k nalezeni rootu
		mtab = open('/etc/mtab')
		rootdev = None
		devstrings = []
		require = []
		processed = []
		for line in mtab:
			spl = line.strip().split()
			if spl[1] == '/mnt/target':
				rootdev = spl[0]
				break
		mtab.close()
		if rootdev == None:
			print "unable to find root device"
		else:
			require.append(rootdev)

		while len(require) > 0:
			mydev = require.pop()
			# je to lvm?
			try:
				lv_name,vg_name = subprocess.check_output(['lvm','lvs','-o','lv_name,vg_name','--noheadings',mydev], stderr=open('/dev/null','w')).strip().split()
				devstrings.append('rd.lvm.lv=%s/%s' % (vg_name,lv_name))
				# require PVs for VG
				if ('VG---'+vg_name) not in processed:
					processed.append('VG---'+vg_name)
					pvs = subprocess.check_output(['lvm','vgs','-o','pv_name','--noheadings',vg_name], stderr=open('/dev/null','w')).strip().split('\n')
					for pv in pvs:
						require.append(pv.strip())
				continue
			except:
				pass
			try:
				if mydev in processed:
					continue
				processed.append(mydev)
				mdlines = subprocess.check_output(['mdadm','-D',mydev,'-Y'], stderr=open('/dev/null','w')).strip().split('\n')
				for line in mdlines:
					spl = line.split('=')
					if spl[0] == 'MD_UUID':
						devstrings.append('rd.md.uuid=%s' % (spl[1],))
					elif re.match('^MD_DEVICE.*_DEV', spl[0]):
						require.append(spl[1])
				continue
			except:
				pass
	
			# standard partition
			# print mydev
			pass

		fd = open('/mnt/target/boot/grub2/grub.cfg')
		grub_cfg = fd.readlines()
		fd.close()
		for i in range(len(grub_cfg)):
			spl = grub_cfg[i].strip().split()
			if spl == []:
				continue
			if spl[0] == 'linux16':
				appends = list(devstrings)
				for item in appends:
					if item in spl:
						appends.remove(item)
				grub_cfg[i] = grub_cfg[i].rstrip()+' '+' '.join(appends)+'\n'
		fd = open('/mnt/target/boot/grub2/grub.cfg','w')
		fd.writelines(grub_cfg)
		fd.close()

		# bootloader end


		installation_status = {'action':'running_post','progress':{'current':0,'total':len(config.ks['post'])}}
		if status_updater:
			status_updater.update_status(installation_status)
		if _gui:
			_gui.update_progress(installation_status)
		print 'running post-install scripts...'

		for post in config.ks['post']:
			postscript = tempfile.NamedTemporaryFile(delete=False,dir='/mnt/target/tmp')
			postscript.write(post['buffer'])
			postscript.close()
			os.system('chroot /mnt/target sh {0} >{0}.log 2>&1'.format(postscript.name[len('/mnt/target'):]))
			os.unlink(postscript.name)
			installation_status['progress']['current'] +=1
			if status_updater:
				status_updater.update_status(installation_status)
			if _gui:
				_gui.update_progress(installation_status)
		
		# clean transaction files
		os.system('rm -f /mnt/target/var/lib/yum/transaction-*')
		
		if(config.public_installation != False):
			installation_status = {'action':'pending_reboot','progress':None}
			if status_updater:
				status_updater.update_status(installation_status,time.time)
				
			status_updater.stop()
			status_updater.join()
			installation_status = {'action':'rebooting','progress':None}
			if api:
				api.installation_update_status(installation_status)
				
			os.system('umount /mnt/target/* /mnt/target')
			os.system('reboot')
		
		elif config.ks['reboot'] and not config.no_reboot:
			installation_status = {'action':'pending_reboot','progress':None}
			if status_updater:
				status_updater.update_status(installation_status,time.time)
			if _gui:
				_gui.update_progress(installation_status)
				_gui.show_reboot_dialog()
				while not config.reboot_button_pressed:
					time.sleep(0.1)
			else:
				print "Installation finished. Please remove installation media."
				print "Press return to reboot..."
				sys.stdin.readline()
			
			status_updater.stop()
			status_updater.join()
			
			installation_status = {'action':'rebooting','progress':None}
			if api:
				api.installation_update_status(installation_status)
			if _gui:
				_gui.update_progress(installation_status)
			os.system('umount /mnt/target/* /mnt/target')
			os.system('reboot')
		else:
			status_updater.stop()
			status_updater.join()
			
			installation_status = {'action':'finished','progress':None}
			if api:
				api.installation_update_status(installation_status)
			if _gui:
				_gui.update_progress(installation_status)
			print 'target system is mounted in /mnt/target. Unmount before rebooting...'
			if _gui:
				_gui.close()
	
	except SystemError as ex:
		if(ex.message == 'unsupported_cpu'):
			data = {
				'title': 'Unsupported CPU',
				'desc': 'CPU doesn\'t support virtualization',
				'type': 'system_error',
				'simple': True
			}
			if(_gui):
				_gui.show_error_dialog(data)
				
			else:
				terminal_error(data)
			
		elif(ex.message == 'unknown_cpu'):
			data = {
				'title': 'Unknown CPU',
				'desc': 'CPU is not supported',
				'type': 'system_error',
				'simple': True
			}
			if(_gui):
				_gui.show_error_dialog(data)
				
			else:
				terminal_error(data)
	
	except Exception as ex:
		#default error
		tb = traceback.extract_tb(sys.exc_info()[2],1)
		data = {
			'title': 'Unexpected error',
			'desc': 'File: {}:{} in {}\nError: {}\n\nStack:\n{}'.format(tb[0][0], tb[0][1], tb[0][2], ex.message, traceback.format_exc()),
			'type': 'exception'
		}
		if(_gui):
			_gui.show_error_dialog(data)
			
		else:
			terminal_error(data)
			
			
def terminal_error(data):
	print data['title']
	print '\n', data['desc']
	if(data['type'] in ['system_error', 'exception']):
		print '\npress any key to reboot...'
		getch()

def getch():
	fd = sys.stdin.fileno()
	old = termios.tcgetattr(fd)
	try:
		tty.setraw(fd)
		return sys.stdin.read(1)
		
	finally:
		termios.tcsetattr(fd, termios.TCSADRAIN, old)


def term(signum, frame):
	"""docstring for terminate"""
	global _gui
	
	_gui.app.exit(1)

if __name__ == '__main__':
	config.parse_cmdline()
	
	if config.api_base != None:
		ADMINISTRACE_BASE = config.api_base
	
	if config.text_mode:
		install()
		sys.exit()
	else:
		_gui = gui.Gui(sys.argv)
	
		install_thread = threading.Thread(target=install,args=(_gui,))
		install_thread.daemon = True
		install_thread.start()
	
	
		# signal.signal(signal.SIGTERM, term)
		signal.signal(signal.SIGINT, term)
	
		_gui.exec_()
		sys.exit()
